@@ -1,5 +1,5 @@
 qut <-
-function(y,X,fit,family=gaussian,alpha.level='default',M=1000,qut.standardize=TRUE,intercept=TRUE,offset=NULL,bootstrap=TRUE,sigma=ifelse(n>2*p,'ols','qut'),beta0='iter',estimator='unbiased',type=c('glmnet','lars'),lambda.seq=0,penalty.factor=rep(1,p),lambda.min.ratio=ifelse(n<p,0.01,0.0001),nlambda=100,lambda=NULL,...){
+function(y,X,fit,family=gaussian,alpha.level=0.05,M=1000,qut.standardize=TRUE,intercept=TRUE,offset=NULL,bootstrap=TRUE,sigma=ifelse(n>2*p,'ols','qut'),beta0='iter',estimator='unbiased',type=c('glmnet','lars','flare'),lambda.seq=0,penalty.factor=rep(1,p),lambda.min.ratio=ifelse(n<p,0.01,0.0001),nlambda=ifelse(type=='flare',2,100),lambda=NULL,...){
 #FUNCTION TO FIT GLM WITH THE QUANTILE UNIVERSAL THRESHOLD
 
 	#Hidden function to get glm fit with non-zero coefficients found by LASSO-GLM
@@ -7,11 +7,10 @@ function(y,X,fit,family=gaussian,alpha.level='default',M=1000,qut.standardize=TR
 		if(type=='glmnet') betatemp=beta[-1] else betatemp=beta
 		betaglm=betatemp*0
 		converged=TRUE
-		
 		if(is.null(offset)) O=rep(0,n)
 		if(sum(betatemp!=0)>0){
-			if(intercept) outglm=glm(y~X0[,betatemp!=0],family=family,offset=offset)
-			else outglm=glm(y~X0[,betatemp!=0]-1,family=family,offset=offset)
+			if(intercept) outglm=glm(y~as.matrix(X0[,betatemp!=0]),family=family,offset=offset)
+			else outglm=glm(y~as.matrix(X0[,betatemp!=0])-1,family=family,offset=offset)
 			betaglm0=outglm$coef
 			converged=outglm$converged
 		}
@@ -46,6 +45,20 @@ function(y,X,fit,family=gaussian,alpha.level='default',M=1000,qut.standardize=TR
 	
 	#Check for warnings
 	type=match.arg(type)
+	if(type=='flare'){
+		if(intercept==FALSE){
+			stop('Square root lasso is based on flare package and requires an intercept. Try setting intercept=TRUE')
+		}
+		if(!is.null(no.penalty)){
+			stop('Square root lasso is based on flare package and does not allow no penalty factors of zero')
+		}
+		if(!is.null(offset)){
+			stop('Square root lasso is based on flare package and does not allow no penalty subsets. Try setting offset=NULL')
+		}
+		if(f$family!='gaussian'){
+			stop('Square root lasso is for Gaussian family')
+		}
+	}
 	if(type=='lars'&f$family!='gaussian'){
 		warning("type lars is just for gaussian family; set to glmnet")
 		type='glmnet'
@@ -60,22 +73,31 @@ function(y,X,fit,family=gaussian,alpha.level='default',M=1000,qut.standardize=TR
 	}
 	
 	#Perform one iteration to estimate beta0 when family is not Gaussian, and there is an intercept and/or there are unpenalized columns, and beta0 is unknown
-	if(beta0[1]=='iter'&!(!intercept&is.null(no.penalty))&f$family!='gaussian'){
+	if((beta0[1]=='iter'|beta0[1]=='iterglm')&!(!intercept&is.null(no.penalty))&f$family!='gaussian'){
 		#get qut estimates with beta0=glm(y~A)
 		outqut=qut(y=y,X=X,fit=fit,family=family,alpha.level=alpha.level,M=M,qut.standardize=qut.standardize,
 			intercept=intercept,offset=offset,bootstrap=bootstrap,sigma=sigma,beta0='noiter',estimator=estimator,
 			type=type,lambda.seq=lambda.seq,penalty.factor=penalty.factor,lambda.min.ratio=lambda.min.ratio,nlambda=nlambda,lambda=lambda,...)	
 		#Get new value for beta0
 		if(intercept) indbeta0=c(1,no.penalty+1) else indbeta0=no.penalty+1
-		beta0=outqut$beta[indbeta0]
+		if(beta0[1]=='iter') beta0=outqut$beta[indbeta0]
+		else if (beta0[1]=='iterglm'){
+			if(attr(outqut$betaglm,'converged')==TRUE) beta0=outqut$betaglm[indbeta0]
+			else beta0=outqut$beta[indbeta0]
+		}
 	}
 	else if(!is.numeric(beta0)) beta0=NA
 	
-    #Obtain lambdaqut (Quantile Universal Threshold)
-	outqut=lambdaqut(y=y,X=X,alpha.level=alpha.level,M=M,qut.standardize=qut.standardize, family=family,intercept=intercept,no.penalty=no.penalty,offset=offset,bootstrap=bootstrap,beta0=beta0)
+    
+	if(type=='flare') method='sqrtlasso'
+	else method='lasso'
+	
+	#Obtain lambdaqut (Quantile Universal Threshold)
+	outqut=lambdaqut(y=y,X=X,alpha.level=alpha.level,M=M,qut.standardize=qut.standardize, family=family,intercept=intercept,no.penalty=no.penalty,offset=offset,bootstrap=bootstrap,beta0=beta0,method=method)
 	X=outqut$Xnew #Get standardized matrix
 	lambdaqut=outqut$lambda
-	
+	beta0=outqut$beta0 #Get estimated intercept
+
 	#Check if lambda is Inf: Problems estimating the model
 	if(lambdaqut==Inf){
 		beta=rep(0,n+1)
@@ -83,14 +105,16 @@ function(y,X,fit,family=gaussian,alpha.level='default',M=1000,qut.standardize=TR
 		beta[1]=NA
 		betaglm[1]=NA
 		warning('Difficulty estimating this model, try increasing M')
-		if(type=='glmnet') lambdamax=outqut$lambda.max*sum(penalty.factor)/p
+		
+		if(type=='flare') lambdamax=outqut$lambda.max
+		else if(type=='glmnet') lambdamax=outqut$lambda.max*sum(penalty.factor)/p
 		else lambdamax=outqut$lambda.max*n
 	}
 	
 	#No problems with lambda
 	else{
 		#Estimate sigma for gaussian family (if not specified)
-		if(f$family=='gaussian'){
+		if(f$family=='gaussian'&type!='flare'){
 			if(!is.numeric(sigma)){
 				if(sigma=='ols'){
 					if(n<p){
@@ -122,12 +146,7 @@ function(y,X,fit,family=gaussian,alpha.level='default',M=1000,qut.standardize=TR
 		if(type=='glmnet'){ #GLMNET
 			lambdamax=outqut$lambda.max*sum(penalty.factor)/p
 			#set sequence of lambdas
-			if(lambda.seq==2){  #default glmnet values
-				if(missing(lambda.min.ratio)) lambda.min.ratio=ifelse(n<p,0.01,0.0001)
-				if(missing(nlambda)) nlambda=100
-				if(missing(lambda)) lambda=NULL
-			}
-			else{ #starting from lambdaqut to lambdamax
+			if(lambda.seq!=2){ #starting from lambdaqut to lambdamax
 				lambda.min.ratio=lambdaqut*sigma/lambdamax*0.9999
 
 				if(lambda.seq==0 ){ #equi-spaced
@@ -139,15 +158,12 @@ function(y,X,fit,family=gaussian,alpha.level='default',M=1000,qut.standardize=TR
 					else lambda=exp(seq(log(lambdamax),log(lambdaqut*sigma),length=nlambda))/n #n lambdas in logscale
 				}
 			}
-			if(missing(penalty.factor)) penalty.factor=rep(1,p)
-			if(!missing(no.penalty)) penalty.factor[no.penalty]=0
+			penalty.factor[no.penalty]=0
 
 			#Obtain GLM L1 path using GLMNET
 			if(missing(fit)) fit=glmnet(X,y,standardize=FALSE,intercept=intercept,family=f$family,penalty.factor=penalty.factor,offset=offset,nlambda=nlambda,lambda.min.ratio=lambda.min.ratio,lambda=lambda,...)		
-
 			if(max(fit$lambda)==Inf) beta=rep(0,p+1)
 			else beta=coef(fit, s=lambdaqut*sigma/n*sum(penalty.factor)/p,offset=offset)/c(1,outqut$scale.factor) 
-
 			lambdaqut=lambdaqut/n
 
 			#Get GLM fitted coefficients
@@ -158,7 +174,6 @@ function(y,X,fit,family=gaussian,alpha.level='default',M=1000,qut.standardize=TR
 		
 			#LASSO fit
 			lambdamax=outqut$lambda.max*n
-			if(missing(penalty.factor)) penalty.factor=rep(1,p)
 			X=X/penalty.factor
 			
 			if(missing(fit)) fit=lars(X,y,intercept=intercept,normalize=FALSE,...)
@@ -176,6 +191,25 @@ function(y,X,fit,family=gaussian,alpha.level='default',M=1000,qut.standardize=TR
 			names(beta)[-1]=1:p	
 			
 		}
+		
+		else if(type=='flare'){
+			lambdamax=outqut$lambda.max*n
+			X=X/penalty.factor
+			
+			fit=slimsd(X,y,nlambda=nlambda,lambda.min.value=lambdaqut)
+			beta=fit$beta[,nlambda]
+		
+			beta0=mean(y)-mean(X%*%beta)
+			beta=beta/penalty.factor/outqut$scale.factor
+			
+			#Get GLM fitted coefficients
+			betaglm=try(betaGLMestimates(),silent=TRUE)
+			
+			beta=c(beta0,beta)
+
+			names(beta)[1]='(Intercept)'	
+			names(beta)[-1]=1:p	
+		}
 	}
 
 	#OUTPUT
@@ -187,8 +221,9 @@ function(y,X,fit,family=gaussian,alpha.level='default',M=1000,qut.standardize=TR
 	if(class(betaglm)[1]=="try-error") warning("No valid GLM fits were possible to find")
 	else betaglm[is.na(betaglm)]=0
 	out$betaglm=betaglm
-	out$fit=fit
+	if(missing(fit)) out$fit=NA else out$fit=fit
 	out$family=family
+	out$beta0=beta0;
 	if(f$family=='gaussian') out$sigma=sigma
 	else out$sigma=NA
 	class(out)='qut'
